package com.fr.plugin.decision.auth.filter;

import Com.Zstar.UUMS.Pub.DesSecurity;
import com.fr.base.ServerConfig;
import com.fr.base.TemplateUtils;
import com.fr.base.email.EmailCenter;
import com.fr.cbb.general.GeneralUtils;
import com.fr.data.NetworkHelper;
import com.fr.decision.authority.AuthorityContext;
import com.fr.decision.authority.data.CustomRole;
import com.fr.decision.authority.data.User;
import com.fr.decision.config.FSConfig;
import com.fr.decision.config.LoginVerificationConfig;
import com.fr.decision.fun.impl.AbstractGlobalRequestFilterProvider;
import com.fr.decision.mobile.terminal.TerminalHandler;
import com.fr.decision.privilege.TransmissionTool;
import com.fr.decision.record.LoginMessage;
import com.fr.decision.webservice.bean.authentication.LoginClientBean;
import com.fr.decision.webservice.bean.authentication.LoginResponseInfoBean;
import com.fr.decision.webservice.bean.authentication.OriginUrlResponseBean;
import com.fr.decision.webservice.utils.ControllerFactory;
import com.fr.decision.webservice.utils.DecisionServiceConstants;
import com.fr.decision.webservice.utils.DecisionStatusService;
import com.fr.decision.webservice.utils.WebServiceUtils;
import com.fr.decision.webservice.utils.controller.AuthenticController;
import com.fr.decision.webservice.v10.login.LoginService;
import com.fr.decision.webservice.v10.password.strategy.PasswordStrategyService;
import com.fr.decision.webservice.v10.sms.SMSService;
import com.fr.decision.webservice.v10.user.UserService;
import com.fr.log.FineLoggerFactory;
import com.fr.plugin.decision.auth.RequestReaderHttpServletRequestWrapper;
import com.fr.plugin.decision.auth.config.AuthLoginPluginConfig;
import com.fr.security.TokenProcessorManager;
import com.fr.stable.StableUtils;
import com.fr.stable.StringUtils;

import javax.servlet.FilterChain;
import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.fr.stable.query.QueryFactory;
import com.fr.stable.web.Device;
import com.fr.store.Converter;
import com.fr.third.org.apache.commons.lang3.time.DateFormatUtils;
import com.fr.third.org.apache.http.HttpStatus;
import com.fr.third.org.apache.http.client.methods.CloseableHttpResponse;
import com.fr.third.org.apache.http.client.methods.HttpPost;
import com.fr.third.org.apache.http.entity.StringEntity;
import com.fr.third.org.apache.http.impl.client.CloseableHttpClient;
import com.fr.third.org.apache.http.impl.client.HttpClients;
import com.fr.third.org.apache.http.util.EntityUtils;
import com.fr.web.utils.WebUtils;
import net.sf.json.JSONObject;

import java.io.PrintWriter;
import java.util.*;

public class AuthLoginRequestFilter extends AbstractGlobalRequestFilterProvider {

    @Override
    public String filterName() {
        return "AuthLoginFilter";
    }

    @Override
    public String[] urlPatterns() {
        return new String[]{
                "/decision/login"
        };
    }

    /**
     * 实现统一身份认证拦截处理
     * @param req
     * @param res
     * @param filterChain
     */
    @Override
    public void doFilter(HttpServletRequest req, HttpServletResponse res, FilterChain filterChain) {

        try {

            // 获取请求中的流如何，将取出来的字符串，再次转换成流，然后把它放入到新request对象中。
            RequestReaderHttpServletRequestWrapper requestWrapper
                    = new RequestReaderHttpServletRequestWrapper((HttpServletRequest) req);

            String body = requestWrapper.getBody();

            // 验证请求
            boolean bolRet = true;

            if (StringUtils.isNotEmpty(body)) {

                JSONObject bodyJson = JSONObject.fromObject(body);

                // 用户名
                String username = bodyJson.getString("username");

                // 系统管理员 保留帆软验证
                if ("admin".equals(username)) {
                    filterChain.doFilter(requestWrapper, res);
                    return;
                }

                // 加密后密码
                String encPassword = bodyJson.getString("password");

                String origin = GeneralUtils.objectToString(bodyJson.get("origin"));

                int validity = bodyJson.getInt("validity");

                // 帆软解密密码
                String password = TransmissionTool.decrypt(encPassword);

                // 统一身份认证
                String responseStr = getUserInfo(username, password, "tzbank");
                if (responseStr == null) {
                    responseStr = getUserInfo(username, password, "czbank");
                }

                if (responseStr != null) {

                    JSONObject resData = JSONObject.fromObject(responseStr);

                    if ("S".equals(resData.getJSONObject("SYS_HEAD").getString("RET_STATUS"))
                            && "000000".equals(resData.getJSONObject("SYS_HEAD").getJSONArray("RET").getJSONObject(0).getString("RET_CODE"))) {

                        UserService userService = UserService.getInstance();
                        User user = userService.getUserByUserName(username);

                        if (user != null) {
                            String frToken = LoginService.getInstance().login(requestWrapper, res, username);
                            requestWrapper.setAttribute(DecisionServiceConstants.FINE_AUTH_TOKEN_NAME, frToken);

                            // 以下部分参考 LoginService.login 方法 START
                            OriginUrlResponseBean url = null;
                            if (StringUtils.isNotEmpty(origin)) {
                                url = (OriginUrlResponseBean) DecisionStatusService.originUrlStatusService().get(origin);
                                DecisionStatusService.originUrlStatusService().delete(origin);
                                if (url == null) {
                                    url = new OriginUrlResponseBean(TemplateUtils.render("${fineServletURL}"));
                                }
                            } else {
                                url = new OriginUrlResponseBean(TemplateUtils.render("${fineServletURL}"));
                            }

                            Device device = NetworkHelper.getDevice(req);
                            String ip = WebServiceUtils.getIpInfoFromRequest(req);

                            TerminalHandler terminal = TerminalHandler.getTerminal(req, device);
                            AuthenticController authenticController = ControllerFactory.getInstance().getAuthenticController(user.getId());
                            long tokenTimeout = this.getTokenTimeOutByValidity(validity);
                            String token = this.generateToken(user, tokenTimeout);
                            authenticController.verifySingleLoginStatus(user.getUserName(), terminal, token);
                            if (authenticController.passwordChangeable(user)) {
                                PasswordStrategyService.getInstance().checkPasswordNeedUpdate(user, token);
                            }

                            LoginClientBean clientBean = new LoginClientBean(req, device, terminal);
                            clientBean.setUsername(user.getUserName());
                            clientBean.setToken(token);
                            clientBean.setValidity(validity);
                            clientBean.setUserId(user.getId());
                            authenticController.logoutSingleLoginInvalidUser(user.getUserName(), terminal);
                            this.addLoginStatus(token, clientBean, tokenTimeout);
                            LoginService.getInstance().checkServerInitStatus();
                            this.createLoginMessage(ip, user.getUserName(), user.getId());
                            if (ServerConfig.getInstance().isTokenFromCookie()) {
                                this.writeToken2Cookie(res, token, validity);
                            }

                            LoginResponseInfoBean responseInfoBean = new LoginResponseInfoBean(token, url, user.getUserName(), validity);
                            // 以上部分参考 LoginService.login 方法 END

                            com.fr.json.JSONObject data = buildResponseValue(responseInfoBean);
                            WebUtils.printAsJSON(res, data);

                            return;

                        } else {
                            bolRet = false;
                        }
                    } else {
                        bolRet = false;
                    }

                } else {
                    bolRet = false;
                }
            }

            if (!bolRet) {
                // 统一身份认证请求失败  用户名或密码错误
                JSONObject object = new JSONObject();
                PrintWriter pw;
                object.put("errorCode", "21300007");
                object.put("errorMsg", "User not exist, or wrong password!");
                pw = WebUtils.createPrintWriter(res);
                res.setContentType("application/json;charset=utf-8");
                String result = object.toString();
                pw.println(result);
                pw.flush();
                pw.close();

                return;
            }

            filterChain.doFilter(requestWrapper, res);

        } catch (Exception e) {
            e.printStackTrace();
            FineLoggerFactory.getLogger().error(e.getMessage(), e);
        }
    }

    private String getUserInfo(String username, String password, String bankType) {

        // 配置参数
        AuthLoginPluginConfig authLoginPluginConfig = AuthLoginPluginConfig.getInstance();

        // 服务代码
        String serviceCode = authLoginPluginConfig.getServiceCode();
        // 服务场景
        String serviceScene = authLoginPluginConfig.getServiceScene();
        // 消费系统ID
        String consumerId = authLoginPluginConfig.getConsumerId();
        // 消费顺序号码
        String consumerSeqNo = authLoginPluginConfig.getConsumerSeqNo();
        // 系统编码
        String systemNo = authLoginPluginConfig.getSystemNo();
        // 请求URL
        String postUrl = authLoginPluginConfig.getUrl();

        Req request = new Req();
        SYS_HEAD sysHeader = new SYS_HEAD();
        APP_HEAD appHeader = new APP_HEAD();
        Certificate_BODY BODY = new Certificate_BODY();

        request.setSYS_HEAD(sysHeader);
        request.setAPP_HEAD(appHeader);
        request.setBODY(BODY);

        String yyyyMMddHHmmssSSS = DateFormatUtils.format(System.currentTimeMillis(), "yyyyMMddHHmmssSSS");

        sysHeader.setCONSUMER_ID(consumerId);
        sysHeader.setCONSUMER_SEQ_NO(consumerSeqNo);
        sysHeader.setSERVICE_CODE(serviceCode);
        sysHeader.setSERVICE_SCENE(serviceScene);
        sysHeader.setTRAN_DATE(yyyyMMddHHmmssSSS.substring(0, 8));
        sysHeader.setTRAN_TIMESTAMP(yyyyMMddHHmmssSSS.substring(8, 17));

        BODY.setSYSTEM_NO(systemNo);
        BODY.setBANK_SYSTEM_TYPE(bankType);

        BODY.setUSER_ID(username);
        DesSecurity desSecurity = new DesSecurity();
        BODY.setUSER_PASSWORD(desSecurity.pinEncrypt(password));

        sysHeader.setUSER_ID(username);
        DesSecurity ds = new DesSecurity();
        sysHeader.setUSER_PASSWORD(ds.pinEncrypt(password));

        String reqStr = JSONObject.fromObject(request).toString();
        FineLoggerFactory.getLogger().info("reqStr=" + reqStr);

        StringEntity sn = new StringEntity(reqStr, "utf-8");

        HttpPost post = new HttpPost(postUrl);
        post.addHeader("Content-Type", "application/json");

        post.setEntity(sn);

        CloseableHttpClient httpClient = HttpClients.createDefault();

        CloseableHttpResponse response = null;
        String responseStr = null;

        try {

            response = httpClient.execute(post);

            int statusCode = response.getStatusLine().getStatusCode();
            responseStr = EntityUtils.toString(response.getEntity(), "utf-8");
            FineLoggerFactory.getLogger().info("statusCode=" + statusCode);
            FineLoggerFactory.getLogger().info("responseStr=" + responseStr);

            JSONObject resData = JSONObject.fromObject(responseStr);

            if (statusCode != HttpStatus.SC_OK) {

                responseStr = null;
            } else {

                // 用户不存在
                if ("4444".equals(resData.getJSONObject("SYS_HEAD").getJSONArray("RET").getJSONObject(0).getString("RET_CODE"))) {

                    responseStr = null;
                }
            }

        } catch (Exception e) {
            e.printStackTrace();
            FineLoggerFactory.getLogger().error(e.getMessage(), e);
        }

        return responseStr;
    }

    private long getTokenTimeOutByValidity(int validity) {
        return validity == -2 ? 1209600000L : FSConfig.getInstance().getLoginConfig().getLoginTimeout();
    }

    private String generateToken(User user, long timeOut) {
        return this.generateToken(user.getUserName(), user.getDisplayName(), user.getTenantId(), timeOut);
    }

    private String generateToken(String username, String displayName, String tenantId, long timeOut) {
        Map<String, Object> claims = new HashMap();
        claims.put("description", displayName);
        claims.put("tenantId", tenantId);
        return TokenProcessorManager.getCurrentProcessor().generateToken(username, claims, timeOut);
    }

    private void addLoginStatus(String token, LoginClientBean clientBean, long tokenTimeout) throws Exception {
        DecisionStatusService.loginStatusService().put(token, clientBean, new Converter<LoginClientBean>() {
            public String[] createAlias(LoginClientBean loginClientBean) {
                return new String[]{loginClientBean.getUsername()};
            }
        }, tokenTimeout);
    }

    private LoginMessage createLoginMessage(String ip, String username, String userId) throws Exception {
        return LoginMessage.build(ip, username, this.getUserRole(userId));
    }

    private boolean needLoginVerification(Device device) {
        if (device.isMobile()) {
            return LoginVerificationConfig.getInstance().isSmsVerification() && SMSService.getInstance().isSMSAvailable();
        } else {
            return LoginVerificationConfig.getInstance().isSmsVerification() && SMSService.getInstance().isSMSAvailable() || LoginVerificationConfig.getInstance().isEmailVerification() && EmailCenter.isDefaultEmailConfigValid();
        }
    }

    private String getUserRole(String userId) throws Exception {
        List<String> customRoleName = new ArrayList();
        List<CustomRole> customRoles = AuthorityContext.getInstance().getCustomRoleController().findByUser(userId, QueryFactory.create());
        if (customRoles != null && !customRoles.isEmpty()) {
            Iterator var4 = customRoles.iterator();

            while(var4.hasNext()) {
                CustomRole customRole = (CustomRole)var4.next();
                customRoleName.add(customRole.getName());
            }
        }

        return customRoleName.isEmpty() ? "" : StableUtils.join(customRoleName, ",");
    }

    private void writeToken2Cookie(HttpServletResponse response, String token, int validity) {
        try {
            if (StringUtils.isNotEmpty(token)) {
                Cookie cookie = new Cookie("fine_auth_token", token);
                long cookieLife = validity == -2 ? 1209600000L : (long)validity;
                cookie.setMaxAge((int)cookieLife);
                cookie.setPath(ServerConfig.getInstance().getCookiePath());
                response.addCookie(cookie);
                Cookie rememberLogin = new Cookie("fine_remember_login", String.valueOf(validity == -2 ? -2 : -1));
                rememberLogin.setMaxAge((int)cookieLife);
                rememberLogin.setPath(ServerConfig.getInstance().getCookiePath());
                response.addCookie(rememberLogin);
            } else {
                FineLoggerFactory.getLogger().error("empty token cannot save.");
            }
        } catch (Exception var8) {
            FineLoggerFactory.getLogger().error(var8.getMessage(), var8);
        }
    }

    private static com.fr.json.JSONObject buildResponseValue(LoginResponseInfoBean responseInfoBean){

        com.fr.json.JSONObject data = new com.fr.json.JSONObject();
        data.put("username",responseInfoBean.getUsername());
        data.put("validity",responseInfoBean.getValidity());
        data.put("callback",responseInfoBean.getCallback());
        data.put("accessToken",responseInfoBean.getAccessToken());

        OriginUrlResponseBean originUrlResponse = responseInfoBean.getOriginUrlResponse();
        JSONObject originUrlJson = new JSONObject();
        originUrlJson.put("originUrl",originUrlResponse.getOriginUrl());
        originUrlJson.put("method",originUrlResponse.getMethod());
        originUrlJson.put("parameters",originUrlResponse.getParameters());

        data.put("originUrlResponse",originUrlJson);
        data.put("url",responseInfoBean.getUrl());

        return new com.fr.json.JSONObject().put("data",data);
    }

    public static class Req<T> {
        private SYS_HEAD SYS_HEAD;
        private APP_HEAD APP_HEAD;
        private T BODY;

        public SYS_HEAD getSYS_HEAD() {
            return SYS_HEAD;
        }

        public void setSYS_HEAD(SYS_HEAD SYS_HEAD) {
            this.SYS_HEAD = SYS_HEAD;
        }

        public APP_HEAD getAPP_HEAD() {
            return APP_HEAD;
        }

        public void setAPP_HEAD(APP_HEAD APP_HEAD) {
            this.APP_HEAD = APP_HEAD;
        }

        public T getBODY() {
            return BODY;
        }

        public void setBODY(T BODY) {
            this.BODY = BODY;
        }
    }

    public static class SYS_HEAD {
        private String SERVICE_CODE;
        private String SERVICE_SCENE;
        private String CONSUMER_ID;
        private String TRAN_DATE;
        private String TRAN_TIMESTAMP;
        private String CONSUMER_SEQ_NO;
        private String USER_ID;
        private String USER_PASSWORD;

        public String getSERVICE_CODE() {
            return SERVICE_CODE;
        }

        public void setSERVICE_CODE(String SERVICE_CODE) {
            this.SERVICE_CODE = SERVICE_CODE;
        }

        public String getSERVICE_SCENE() {
            return SERVICE_SCENE;
        }

        public void setSERVICE_SCENE(String SERVICE_SCENE) {
            this.SERVICE_SCENE = SERVICE_SCENE;
        }

        public String getCONSUMER_ID() {
            return CONSUMER_ID;
        }

        public void setCONSUMER_ID(String CONSUMER_ID) {
            this.CONSUMER_ID = CONSUMER_ID;
        }

        public String getTRAN_DATE() {
            return TRAN_DATE;
        }

        public void setTRAN_DATE(String TRAN_DATE) {
            this.TRAN_DATE = TRAN_DATE;
        }

        public String getTRAN_TIMESTAMP() {
            return TRAN_TIMESTAMP;
        }

        public void setTRAN_TIMESTAMP(String TRAN_TIMESTAMP) {
            this.TRAN_TIMESTAMP = TRAN_TIMESTAMP;
        }

        public String getCONSUMER_SEQ_NO() {
            return CONSUMER_SEQ_NO;
        }

        public void setCONSUMER_SEQ_NO(String CONSUMER_SEQ_NO) {
            this.CONSUMER_SEQ_NO = CONSUMER_SEQ_NO;
        }

        public String getUSER_ID() {
            return USER_ID;
        }

        public void setUSER_ID(String USER_ID) {
            this.USER_ID = USER_ID;
        }

        public String getUSER_PASSWORD() {
            return USER_PASSWORD;
        }

        public void setUSER_PASSWORD(String USER_PASSWORD) {
            this.USER_PASSWORD = USER_PASSWORD;
        }
    }

    public static class APP_HEAD {
    }

    public static class Certificate_BODY {
        private String BANK_SYSTEM_TYPE;
        private String SYSTEM_NO;
        private String USER_ID;
        private String USER_PASSWORD;

        public String getBANK_SYSTEM_TYPE() {
            return BANK_SYSTEM_TYPE;
        }

        public void setBANK_SYSTEM_TYPE(String BANK_SYSTEM_TYPE) {
            this.BANK_SYSTEM_TYPE = BANK_SYSTEM_TYPE;
        }

        public String getSYSTEM_NO() {
            return SYSTEM_NO;
        }

        public void setSYSTEM_NO(String SYSTEM_NO) {
            this.SYSTEM_NO = SYSTEM_NO;
        }

        public String getUSER_ID() {
            return USER_ID;
        }

        public void setUSER_ID(String USER_ID) {
            this.USER_ID = USER_ID;
        }

        public String getUSER_PASSWORD() {
            return USER_PASSWORD;
        }

        public void setUSER_PASSWORD(String USER_PASSWORD) {
            this.USER_PASSWORD = USER_PASSWORD;
        }
    }
}