package com.fr.plugin.decision.auth.filter;

import com.fr.decision.authority.data.User;
import com.fr.decision.fun.impl.AbstractGlobalRequestFilterProvider;
import com.fr.decision.webservice.utils.DecisionServiceConstants;
import com.fr.decision.webservice.v10.login.LoginService;
import com.fr.decision.webservice.v10.user.UserService;
import com.fr.log.FineLoggerFactory;
import com.fr.plugin.decision.auth.config.SsoLoginPluginConfig;
import com.fr.third.org.apache.http.HttpStatus;
import com.fr.third.org.apache.http.client.methods.CloseableHttpResponse;
import com.fr.third.org.apache.http.client.methods.HttpPost;
import com.fr.third.org.apache.http.entity.StringEntity;
import com.fr.third.org.apache.http.impl.client.CloseableHttpClient;
import com.fr.third.org.apache.http.impl.client.HttpClients;
import com.fr.third.org.apache.http.util.EntityUtils;
import com.fr.web.utils.WebUtils;
import net.sf.json.JSONObject;

import javax.servlet.FilterChain;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

public class SsoLoginRequestFilter extends AbstractGlobalRequestFilterProvider {

    @Override
    public String filterName() {
        return "SsoLoginFilter";
    }

    @Override
    public String[] urlPatterns() {
        return new String[]{
                "/decision/ssoLogin"
        };
    }

    /**
     * 实现单点登录拦截处理
     * @param req
     * @param res
     * @param filterChain
     */
    @Override
    public void doFilter(HttpServletRequest req, HttpServletResponse res, FilterChain filterChain) {

        try {

            String url = WebUtils.getOriginalURL(req);

            if (url.contains("decision/ssoLogin")) {

                // token参数值
                String token = req.getParameter("token");

                // 配置参数
                SsoLoginPluginConfig ssoLoginPluginConfig = SsoLoginPluginConfig.getInstance();

                // 服务代码
                String serviceCode = ssoLoginPluginConfig.getServiceCode();
                // 服务场景
                String serviceScene = ssoLoginPluginConfig.getServiceScene();
                // 消费系统ID
                String consumerId = ssoLoginPluginConfig.getConsumerId();
                // 目标系统ID
                String targetSysId = ssoLoginPluginConfig.getTargetSysId();
                // 系统编码
                String systemNo = ssoLoginPluginConfig.getSystemNo();
                // 请求URL
                String postUrl = ssoLoginPluginConfig.getUrl();

                Req<SINGLE_LOGIN_BODY> request = new Req();
                SYS_HEAD sysHeader = new SYS_HEAD();
                APP_HEAD appHeader = new APP_HEAD();
                SINGLE_LOGIN_BODY BODY = new SINGLE_LOGIN_BODY();

                request.setSYS_HEAD(sysHeader);
                request.setAPP_HEAD(appHeader);
                request.setBODY(BODY);

                sysHeader.setCONSUMER_ID(consumerId);
                sysHeader.setSERVICE_CODE(serviceCode);
                sysHeader.setSERVICE_SCENE(serviceScene);
                sysHeader.setTARGET_SYS_ID(targetSysId);

                BODY.setSYSTEM_NO(systemNo);
                BODY.setTOKEN_NO(token);

                String reqStr = JSONObject.fromObject(request).toString();
                FineLoggerFactory.getLogger().info("reqStr=" + reqStr);

                StringEntity sn = new StringEntity(reqStr, "utf-8");

                HttpPost post = new HttpPost(postUrl);
                post.addHeader("Content-Type","application/json");

                post.setEntity(sn);

                CloseableHttpClient httpClient = HttpClients.createDefault();

                CloseableHttpResponse response = httpClient.execute(post);

                int statusCode = response.getStatusLine().getStatusCode();
                String responseStr = EntityUtils.toString(response.getEntity(), "utf-8");
                FineLoggerFactory.getLogger().info("statusCode=" + statusCode);
                FineLoggerFactory.getLogger().info("responseStr=" + responseStr);

                if (statusCode != HttpStatus.SC_OK) {
                    FineLoggerFactory.getLogger().error("TOKEN验证请求失败");
                    WebUtils.printAsString(res, "TOKEN验证请求失败");
                    return;
                }

                JSONObject resData = JSONObject.fromObject(responseStr);

                if ("S".equals(resData.getJSONObject("SYS_HEAD").getString("RET_STATUS"))
                        && "000000".equals(resData.getJSONObject("SYS_HEAD").getJSONArray("RET").getJSONObject(0).getString("RET_CODE"))) {

                    // 用户名
                    String userName = "";

                    if (resData.getJSONObject("BODY").getString("LOGON_ID") != null) {
                        userName = resData.getJSONObject("BODY").getString("LOGON_ID").trim();
                    }

                    FineLoggerFactory.getLogger().info("userName:" + userName);

                    UserService userService = UserService.getInstance();
                    User user = userService.getUserByUserName(userName);

                    if (user != null) {
                        String frToken = LoginService.getInstance().login(req, res, userName);
                        req.setAttribute(DecisionServiceConstants.FINE_AUTH_TOKEN_NAME, frToken);

                        String newUrl = url.substring(0, url.indexOf("/ssoLogin"));
                        res.sendRedirect(newUrl);

                        return;

                    } else {
                        FineLoggerFactory.getLogger().error("获取的用户 :{} 在帆软系统中不存在", userName);
                        WebUtils.printAsString(res, "获取的用户 : " + userName + "在帆软系统中不存在");
                        return;
                    }

                } else {
                    FineLoggerFactory.getLogger().error("TOKEN验证请求失败");
                    WebUtils.printAsString(res, "TOKEN验证请求失败");
                    return;
                }
            }

            filterChain.doFilter(req, res);
        } catch (Exception e) {
            e.printStackTrace();
            FineLoggerFactory.getLogger().error(e.getMessage(), e);
            try {
                WebUtils.printAsString(res, "TOKEN验证请求失败");
            } catch (Exception ex) {
            }
        }
    }

    public static class Req<T> {
        private SYS_HEAD SYS_HEAD;
        private APP_HEAD APP_HEAD;
        private T BODY;

        public SYS_HEAD getSYS_HEAD() {
            return SYS_HEAD;
        }

        public void setSYS_HEAD(SYS_HEAD SYS_HEAD) {
            this.SYS_HEAD = SYS_HEAD;
        }

        public APP_HEAD getAPP_HEAD() {
            return APP_HEAD;
        }

        public void setAPP_HEAD(APP_HEAD APP_HEAD) {
            this.APP_HEAD = APP_HEAD;
        }

        public T getBODY() {
            return BODY;
        }

        public void setBODY(T BODY) {
            this.BODY = BODY;
        }
    }

    public static class SYS_HEAD {
        private String SERVICE_CODE;
        private String SERVICE_SCENE;
        private String CONSUMER_ID;
        private String TARGET_SYS_ID;

        public String getSERVICE_CODE() {
            return SERVICE_CODE;
        }

        public void setSERVICE_CODE(String SERVICE_CODE) {
            this.SERVICE_CODE = SERVICE_CODE;
        }

        public String getSERVICE_SCENE() {
            return SERVICE_SCENE;
        }

        public void setSERVICE_SCENE(String SERVICE_SCENE) {
            this.SERVICE_SCENE = SERVICE_SCENE;
        }

        public String getCONSUMER_ID() {
            return CONSUMER_ID;
        }

        public void setCONSUMER_ID(String CONSUMER_ID) {
            this.CONSUMER_ID = CONSUMER_ID;
        }

        public String getTARGET_SYS_ID() {
            return TARGET_SYS_ID;
        }

        public void setTARGET_SYS_ID(String TARGET_SYS_ID) {
            this.TARGET_SYS_ID = TARGET_SYS_ID;
        }
    }

    public static class APP_HEAD {
    }

    public static class SINGLE_LOGIN_BODY {
        private String TOKEN_NO;
        private String SYSTEM_NO;

        public String getTOKEN_NO() {
            return TOKEN_NO;
        }

        public void setTOKEN_NO(String TOKEN_NO) {
            this.TOKEN_NO = TOKEN_NO;
        }

        public String getSYSTEM_NO() {
            return SYSTEM_NO;
        }

        public void setSYSTEM_NO(String SYSTEM_NO) {
            this.SYSTEM_NO = SYSTEM_NO;
        }
    }

    public static class SINGLE_LOGIN_RES_BODY {
        private String LOGON_ID;
        private String BANK_SYSTEM_TYPE;
        private String PASSWORD;

        public String getLOGON_ID() {
            return LOGON_ID;
        }

        public void setLOGON_ID(String LOGON_ID) {
            this.LOGON_ID = LOGON_ID;
        }

        public String getBANK_SYSTEM_TYPE() {
            return BANK_SYSTEM_TYPE;
        }

        public void setBANK_SYSTEM_TYPE(String BANK_SYSTEM_TYPE) {
            this.BANK_SYSTEM_TYPE = BANK_SYSTEM_TYPE;
        }

        public String getPASSWORD() {
            return PASSWORD;
        }

        public void setPASSWORD(String PASSWORD) {
            this.PASSWORD = PASSWORD;
        }
    }
}